﻿using ProjectAirBomber.Entities;

namespace ProjectAirBomber.Drawnings;

public class DrawningBomber
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityBomber? EntityBomber { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки бомбардировщика
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки бомбардировщика
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки бомбардировщика
    /// </summary>
    private readonly int _drawningAirBomberWidth = 190;

    /// <summary>
    /// Высота прорисовки бомбардировщика
    /// </summary>
    private readonly int _drawningAirBomberHeight = 160;

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;
    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;
    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningAirBomberWidth;
    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningAirBomberHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>
    public DrawningBomber()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>

    public DrawningBomber(int speed, double weight, Color bodyColor) : this()
    {
        EntityBomber = new EntityBomber(speed, weight, bodyColor);
    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningAirBomberWidth">Ширина прорисовки бомбардировщика</param>
    /// <param name="drawningAirBomberHeight">Высота прорисовки бомбардировщика</param>

    protected DrawningBomber(int drawningAirBomberWidth, int drawningAirBomberHeight) : this()
    {
        _drawningAirBomberWidth = drawningAirBomberWidth;
        _drawningAirBomberHeight = drawningAirBomberHeight;
    }

    /// <summary>
    /// Конструктор для Extention
    /// </summary>
    /// <param name="airplan"></param>
    public DrawningBomber(EntityBomber bomber) : this()
    {
        EntityBomber = new EntityBomber(bomber.Speed, bomber.Weight, bomber.BodyColor);
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>

    public bool SetPictureSize(int width, int height)
    {
        if (width >= _drawningAirBomberWidth || height >= _drawningAirBomberHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                SetPosition(_startPosX.Value, _startPosY.Value);
            }
            return true;
        }
        return false;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        if (x < 0)
        {
            x = 0;
        }
        else if (x > _pictureWidth - _drawningAirBomberWidth)
        {
            x = _pictureWidth.Value - _drawningAirBomberWidth;
        }

        if (y < 0)
        {
            y = 0;
        }
        else if (y > _pictureHeight - _drawningAirBomberHeight)
        {
            y = _pictureHeight.Value - _drawningAirBomberHeight;
        }

        _startPosX = x;
        _startPosY = y;
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityBomber == null || !_startPosX.HasValue ||
            !_startPosY.HasValue)
        {
            return false;
        }
        if (_startPosX.Value < 0 || _startPosY.Value < 0 || _startPosX > _pictureWidth - _drawningAirBomberWidth || _startPosY > _pictureHeight - _drawningAirBomberHeight)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityBomber.Step > 0)
                {
                    _startPosX -= (int)EntityBomber.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityBomber.Step > 0)
                {
                    _startPosY -= (int)EntityBomber.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityBomber.Step < _pictureWidth - _drawningAirBomberWidth)
                {
                    _startPosX += (int)EntityBomber.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityBomber.Step < _pictureHeight - _drawningAirBomberHeight)
                {
                    _startPosY += (int)EntityBomber.Step;
                }
                return true;
            default:
                return false;
        }
    }

      /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityBomber == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);

        Brush brBlack = new SolidBrush(Color.Black);
        //границы бомбардировщика      
        g.DrawRectangle(pen, _startPosX.Value + 40, _startPosY.Value + 55, 150, 30);

        //Крылья
        Brush brGray = new SolidBrush(Color.Gray);
        g.FillRectangle(brGray, _startPosX.Value + 95, _startPosY.Value, 15, 160);

        //правое крыло
        g.FillPolygon(brGray, new Point[]
        {
            new Point(_startPosX.Value + 95, _startPosY.Value),
            new Point(_startPosX.Value + 110, _startPosY.Value + 5),
            new Point(_startPosX.Value + 118, _startPosY.Value + 85)
        });

        ////левое крыло
        g.FillPolygon(brGray, new Point[]
        {
            new Point(_startPosX.Value + 95, _startPosY.Value + 85),
            new Point(_startPosX.Value + 110, _startPosY.Value + 135),
            new Point(_startPosX.Value + 115, _startPosY.Value + 85)
        });

        //задние крылья

        g.FillRectangle(brGray, _startPosX.Value + 160, _startPosY.Value + 40, 30, 60);

        g.FillPolygon(brGray, new Point[]
        {
            new Point(_startPosX.Value + 160, _startPosY.Value + 40),
            new Point(_startPosX.Value + 190, _startPosY.Value + 55),
            new Point(_startPosX.Value + 190, _startPosY.Value + 5)
        });
        g.FillPolygon(brGray, new Point[]
        {
            new Point(_startPosX.Value + 160, _startPosY.Value + 100),
            new Point(_startPosX.Value + 190, _startPosY.Value + 100),
            new Point(_startPosX.Value + 190, _startPosY.Value + 140)
        });


        //фюзеляж
        Brush br = new SolidBrush(EntityBomber.BodyColor);
        g.FillRectangle(br, _startPosX.Value + 40, _startPosY.Value + 55, 150, 30);

        //передняя часть (треугольник)

        g.FillPolygon(brBlack, new Point[]
        {
            new Point(_startPosX.Value + 40, _startPosY.Value + 55),
            new Point(_startPosX.Value + 40, _startPosY.Value + 85),
            new Point(_startPosX.Value, _startPosY.Value + 65)
        });
    }
}